// CS370 - Fall 2014
// Lab11 - SimpleGLSL

#ifdef OSX
	#include <GLUT/glut.h>
#else
	#include <GL/glew.h>
	#include <GL/glut.h>
#endif
#include <stdio.h>
#include <stdlib.h>

// Shader file utility functions
#include "shaderutils.h"

// Shader files
#define DEFAULT_SHADER 0
#define USER_SHADER 1
GLchar* vertexFile = "simplevert.vs";
GLchar* fragmentFile = "simplefrag.fs";

// Shader objects
GLuint shaderProg;

// TODO: Global shader variables
GLfloat app_time = 0.0f;

int shader_flag = USER_SHADER;

void display();
void render_Scene();
void keyfunc(unsigned char key, int x, int y);
void reshape(int w, int h);
void idlefunc();

int main(int argc, char *argv[])
{
	// Initialize GLUT
	glutInit(&argc,argv);

	// Initialize the window with double buffering and RGB colors
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	// Set the window size to image size
	glutInitWindowSize(500,500);

	// Create window
	glutCreateWindow("Simple GLSL");

#ifndef OSX
	// Initialize GLEW - MUST BE DONE AFTER CREATING GLUT WINDOW
	glewInit();
#endif

	// Define callbacks
	glutDisplayFunc(display);
	glutKeyboardFunc(keyfunc);
	glutReshapeFunc(reshape);
	glutIdleFunc(idlefunc);

	// Set background color to white
	glClearColor(1.0f,1.0f,1.0f,1.0f);

	// Enable depth test
	glEnable(GL_DEPTH_TEST);

	// Load shader programs
	shaderProg = load_shaders(vertexFile,fragmentFile);

	// Activate shader program
	glUseProgram(shaderProg);

	// TODO: Associate time uniform shader variable

	// Begin event loop
	glutMainLoop();

	return 0;
}

// Display callback
void display()
{
	// Reset background
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Set modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(1.0,1.0,1.0,0.0,0.0,0.0,0.0,1.0,0.0);

	// Render scene without shadows
	render_Scene();

	// Flush buffer
	glFlush();

	// Swap buffers
	glutSwapBuffers();
}

// Scene render function
void render_Scene()
{
	// TODO: Attach time uniform shader variable

	// Draw yellow cube
	glPushMatrix();
		glColor3f(1.0f,1.0f,0.0f);
		glutSolidCube(1.0);
	
		// Draw edges
		glColor3f(0.0f,0.0f,0.0f);
		glutWireCube(1.0);
	glPopMatrix();
}

// Keyboard callback
void keyfunc(unsigned char key, int x, int y)
{
	// <space> toggles default shader
	if (key == ' ')
	{
		if (shader_flag == USER_SHADER)
		{
			// Use default shader
			glUseProgram(0);
			shader_flag = DEFAULT_SHADER;
		}
		else if (shader_flag == DEFAULT_SHADER)
		{
			// Use user shader
			glUseProgram(shaderProg);
			shader_flag = USER_SHADER;
		}
	}

	// <esc> quits
	if (key == 27)
	{
		exit(0);
	}

	// Redraw screen
	glutPostRedisplay();
}

// Reshape callback
void reshape(int w, int h)
{
	GLfloat ratio;

	// Set new screen extents
	glViewport(0,0,w,h);

	// Set 3D projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	// Taller than wide so scale height
	if (w <= h)
	{
		ratio = (GLfloat) h / (GLfloat) w;
		glOrtho(-3.0f,3.0f,-3.0f*ratio,3.0f*ratio,-3.0f,3.0f);
	}
	// Wider than tall so scale width
	else
	{
		ratio = (GLfloat) w / (GLfloat) h;
		glOrtho(-3.0f*ratio,3.0*ratio,-3.0f,3.0f,-3.0f,3.0f);
	}
}

// Idle callback
void idlefunc()
{
	// Get time (in sec)
	app_time = glutGet(GLUT_ELAPSED_TIME) / 1000.0f;

	// Render scene
	glutPostRedisplay();
}
